import argparse
import os

from .static_tracker import locate_references, to_markdown

parser = argparse.ArgumentParser()
parser.add_argument(
    "-i",
    "--input",
    default=".",
    type=str,
    help="Target file or folder to analyse. Default: Current directory.",
)
parser.add_argument(
    "-o",
    "--output",
    default="references.md",
    type=str,
    help="File to save the references into. Default: references.md.",
)

args = parser.parse_args()

if os.path.isdir(args.output):
    msg = f"Output must be file, not a directly. Now it is {args.output}"
    raise IsADirectoryError(msg)

to_markdown(locate_references(args.input), args.output)



def fun_with_reference() -> None:
    from r2t2 import science_reference

    science_reference("doing something smart", "My Awesome Book, by me.")
    science_reference(
        "doing something smart in two lines", "Another Awesome Book, by me, 2019"
    )


def test_track_science():
    from r2t2 import track_science
    from r2t2.r2t2 import bibliography

    fun_with_reference()
    assert "My Awesome Book, by me." not in bibliography

    track_science()
    fun_with_reference()
    assert "My Awesome Book, by me." in bibliography


def test_print_references(capsys):
    from r2t2 import track_science, print_references

    track_science()
    fun_with_reference()
    print_references()
    captured = capsys.readouterr()

    assert "[1] doing something smart - fun_with_reference()" in captured.out



from pathlib import Path
from collections import OrderedDict
import re
from inspect import getmodulename
from typing import Union
import os


def locate_references(path: Union[Path, str]) -> OrderedDict:
    """Locates science_reference in path.

    It looks recursively for science_reference markers, taking note of the module, line
    short_purpose and actual reference string.

    Returns
        An Order dict where each key corresponds to a file name with references and each
        value is a list with the reference information as
        [line number, short_purpose, reference]
    """
    filenames = sorted(Path(path).rglob("*.py"))

    ref: OrderedDict = OrderedDict()
    sci_ref = re.compile(r"science_reference\((.*?)\)")
    args = re.compile(r"\"(.*?)\"")

    for filename in filenames:
        ref[filename] = []
        code_str = []

        with open(filename, "r") as f:
            for num, line in enumerate(f):
                if "science_reference(" in line:
                    ref[filename].append([num])

                code_str.append(line.strip())

            single = "".join(code_str)

        results = sci_ref.findall(single)
        for i, ref_raw in enumerate(results):
            if '"' not in ref_raw:
                continue
            ref[filename][i].extend(args.findall(ref_raw))

        if len(ref[filename]) <= 1:
            del ref[filename]

    return ref


def to_markdown(references: OrderedDict, mdfile: Union[Path, str]) -> None:
    """Converts the references dictionary into a markdown file."""
    template = (
        "{id}. [{module}]({filename}) - Line={line}  \n"
        "\tShort purpose: {short}  \n"
        "\tReference: {reference}  \n\n"
    )

    id = 1
    with open(mdfile, mode="w") as f:
        for path, refs in references.items():
            module = getmodulename(path)
            filename = os.path.relpath(path, mdfile)
            for i, ref in enumerate(refs):
                f.write(
                    template.format(
                        id=id,
                        module=module,
                        filename=filename,
                        line=ref[0],
                        short=ref[1],
                        reference=ref[2],
                    )
                )
                id += 1


if __name__ == "__main__":
    path = Path("/Users/dalonsoa/Documents/Projects/R2Tracker/R2T2/")
    mdfile = Path("references.md")

    data = locate_references(path)
    to_markdown(data, mdfile)



import inspect

bibliography: list = []
bibliography_id: list = []
science_tracking: list = []
track_science_references: bool = False
track_each_reference_call: bool = False


def hexID(obj: str) -> str:
    return "{}".format(id(obj))


def science_reference(short_purpose: str, reference: str) -> None:
    """Marker acting as a reference for the origin of specific information

    Acts as a marker in code where particular alogrithms/data/... originates.
    General execution of code silently passes these markers, but remembers how and where
    they were called. Which markers were passed in a particular program run
    can be recalled with print_references().

    Arguments:
    short_purpose: Identify the thing being referenced (string)
    reference: The reference itself, in any sensible format.
    """
    global bibliography, bibliography_id, science_tracking, track_science_references
    if not track_science_references:
        return
    stack = inspect.stack()
    frame, path, line, function, context, index = stack[1]
    del stack

    arguments = inspect.formatargvalues(*inspect.getargvalues(frame))

    availableObjects = {}
    availableObjects.update(frame.f_globals)
    availableObjects.update(frame.f_locals)
    del frame

    func_id = hexID(availableObjects[function])
    identifier = "{} [{}]".format(function, func_id)
    call_record = "{}{}".format(function, arguments)

    add_anyway = False
    if identifier not in bibliography_id:
        bibliography_id.append(identifier)
        bibliography.append(reference)
        add_anyway = True
    if track_each_reference_call or add_anyway:
        science_tracking.append(
            (call_record, short_purpose, bibliography_id.index(identifier) + 1)
        )


def print_references() -> None:
    """ recall the science_reference markers passed, print out the referenes."""
    global bibliography, bibliography_id, science_tracking, track_each_reference_call

    "List of references encountered while executing"
    if not track_each_reference_call:
        print(
            "showing first call only - override with"
            "solcore.track_science(track_each_call=True)"
        )

    for record, purpose, index in science_tracking:
        print("[{}] {} - {}".format(index, purpose, record))
    print()
    for i, b in enumerate(bibliography):
        print("[{}] - {}".format(i + 1, b))


def track_science(track_each_call: bool = False):
    """configure science references -- determine whether or not each call separately or
    only the first for each reference."""
    global track_science_references, track_each_reference_call
    track_science_references = True
    track_each_reference_call = track_each_call



from setuptools import setup, find_packages

r2t2_classifiers = [
    "Development Status :: 3 - Alpha",
    "Programming Language :: Python :: 3",
    "Operating System :: OS Independent",
    "License :: OSI Approved :: MIT License",
    "Topic :: Utilities",
]

tests_require = ["pytest", "pytest-flake8", "pytest-mypy"]
precommit = ["pre-commit"]

setup(
    name="R2T2",
    version="0.1.1",
    author="Research Computing Service, Imperial College London",
    author_email="rcs-support@imperial.ac.uk",
    url="https://github.com/ImperialCollegeLondon/R2T2",
    tests_require=tests_require,
    extras_require={"dev": tests_require + precommit},
    packages=find_packages("."),
    description="Research references tracking tool",
    license="MIT",
    classifiers=r2t2_classifiers,
    python_requires=">=3.0",
)
